@@ -1,4 +1,4 @@
-# $Id: XSV.pm 26 2005-09-22 15:21:37Z zackse $
+# $Id: XSV.pm 31 2005-11-23 15:46:45Z zackse $
 
 package CGI::Application::Plugin::Output::XSV;
 
@@ -25,7 +25,7 @@ our %EXPORT_TAGS= (
   all => [ @EXPORT, @EXPORT_OK ],
 );
 
-our $VERSION= '0.02';
+our $VERSION= '0.03';
 
 ##
 
@@ -150,10 +150,10 @@ sub clean_field_names {
   my $fields= shift;
 
   # using temp var to avoid modifying $fields
+  my @fields_copy= @$fields;
+
   return [
-    map {
-      my $f= $_; $f =~ tr/_/ /; $f =~ s/\b(\w+)/\u$1/g; $f
-    } @$fields
+    map { tr/_/ /; s/\b(\w+)/\u$1/g; $_ } @fields_copy
   ];
 }
 
@@ -216,8 +216,8 @@ You may export all four routines by specifying the export tag C<:all>:
 =head1 PURPOSE
 
 On many websites, I had code to retrieve a list of data items for use
-in an L<HTML::Template(3)|HTML::Template> TMPL_LOOP. Usually this code
-would use the L<DBI(3)|DBI> routine C<fetchall_arrayref( {} )> to get a
+in an L<HTML::Template|HTML::Template(3)> TMPL_LOOP. Usually this code
+would use the L<DBI|DBI(3)> routine C<fetchall_arrayref( {} )> to get a
 list of hash references, one for each data item.
 
   my $users= $sth->fetchall_arrayref( {} );
@@ -249,10 +249,11 @@ almost always looked the same:
     return $output;
 
 The purpose of this module is to provide a simple method, C<xsv_report_web>,
-that wraps the above code while offering enough programmer flexibility.
+that wraps the above code while offering a fair amount of programmer
+flexibility.
 
 For example, the programmer may control the naming of header columns,
-filter each line of output before it is passed to L<Text::CSV_XS(3)|Text::CSV_XS>,
+filter each line of output before it is passed to L<Text::CSV_XS|Text::CSV_XS(3)>,
 and set the filename that is supplied to the user's browser.
 
 Please see the documentation below for C<xsv_report_web> for a list of
@@ -290,16 +291,25 @@ web browser. It sets the content-type header to 'application/x-csv' and sets
 the content-disposition header to 'attachment'.
 
 It should be invoked through a
-L<CGI::Application(3)|CGI::Application> subclass object.
+L<CGI::Application|CGI::Application(3)> subclass object.
 
 It takes a reference to a hash of named parameters. All except for
 C<values> are optional:
 
 =over 8
 
+=item csv_opts
+
+  csv_opts   => { sep_char => "\t" },
+
+A reference to a hash of options passed to the constructor of
+L<Text::CSV_XS|Text::CSV_XS(3)>. The default is an empty hash.
+
 =item fields
 
-A reference to an array of field names or array indices. This parameter
+  fields => [ qw(member_id first_name last_name) ],
+
+A reference to a list of field names or array indices. This parameter
 specifies the order of fields in each row of output.
 
 If C<fields> is not supplied, a list will be generated using the first
@@ -311,12 +321,16 @@ the same as the data provided.
 
 =item filename
 
+  filename => 'members.csv',
+
 The name of the file which will be sent in the HTTP content-disposition
 header. The default is "download.csv".
 
 =item headers
 
-A reference to an array of column headers to be used as the first row
+  headers => [ "Member ID", "First Name", "Last Name" ],
+
+A reference to a list of column headers to be used as the first row
 of the csv report.
 
 If C<headers> is not supplied (and C<include_headers> is not set
@@ -325,30 +339,68 @@ as a parameter to generate column headers.
 
 =item headers_cb
 
+  # replace underscores with spaces
+  headers_cb => sub {
+    my $fields= shift;
+
+    # using temp var to avoid modifying $fields
+    my @fields_copy= @$fields;
+
+    return [
+      map { tr/_/ /; $_ } @fields_copy
+    ];
+  },
+
 A reference to a subroutine used to generate column
 headers from the field names.
 
 A default routine is provided in C<clean_field_names>. This
 function is passed the list of fields (C<fields>) as a parameter
-and should return a reference to an array of column headers.
+and should return a reference to a list of column headers.
 
 =item include_headers
 
+  include_headers => 1,
+
 A true or false value indicating whether to include C<headers>
 (or automatically generated headers) as the first row of output.
 
 The default is true.
 
+=item line_ending
+
+  line_ending     => "\n",
+
+The value appended to each line of csv output. The default is "\n".
+
 =item values
 
-A reference to an array of hash references (such as
-that returned by the L<DBI(3)|DBI> C<fetchall_arrayref( {} )> routine, or
-a reference to an array of list references.
+  values => [
+    { member_id  => 1,
+      first_name => 'Chuck',
+      last_name  => 'Barry', },
+  ],
+
+  # or a list of lists
+  values => [
+    [ 1, 'Chuck', 'Barry', ],
+  ],
+
+A reference to a list of hash references (such as
+that returned by the L<DBI|DBI(3)> C<fetchall_arrayref( {} )> routine, or
+a reference to a list of list references.
 
 This argument is required.
 
 =item get_row_cb
 
+  # uppercase all values -- assumes values are hash references
+  get_row_cb => sub {
+    my( $row, $fields )= @_;
+
+    return [ map { uc } @$row{@$fields} ];
+  },
+
 A reference to a subroutine used to generate each row of output
 (other than the header row). Default routines are provided that
 return each row of C<values> in the order specified by C<headers>.
@@ -359,11 +411,11 @@ This subroutine is passed two parameters for each row:
 
 =item *
 
-the current row (reference to an array)
+the current row (reference to a list of hashes or lists)
 
 =item *
 
-the field list (C<fields> - reference to an array)
+the field list (C<fields> - reference to a list of hash keys or array indices)
 
 =back
 
@@ -389,7 +441,7 @@ the field list (C<fields> - reference to an array)
 
 This function, used internally by C<xsv_report>/C<xsv_report_web>,
 formats a list of values for inclusion a csv file. The return value is
-from C<< $csv->string() >>, where C<$csv> is a L<Text::CSV_XS(3)|Text::CSV_XS> object.
+from C<< $csv->string() >>, where C<$csv> is a L<Text::CSV_XS|Text::CSV_XS(3)> object.
 
 It takes three parameters:
 
@@ -397,7 +449,7 @@ It takes three parameters:
 
 =item *
 
-A L<Text::CSV_XS(3)|Text::CSV_XS> object
+A L<Text::CSV_XS|Text::CSV_XS(3)> object
 
 =item *
 
@@ -409,12 +461,13 @@ The line ending
 
 =back
 
-On an error from L<Text::CSV_XS(3)|Text::CSV_XS>, the function raises an exception.
+On an error from L<Text::CSV_XS|Text::CSV_XS(3)>, the function raises an exception.
 
 On receiving an empty list of values, the function returns the
 line ending only.
 
-XXX should this return a formatted list of empty fields?
+Should this return a formatted list of empty fields? Let me know if you
+think that would be better.
 
 =item B<clean_field_names>
 
@@ -633,14 +686,14 @@ which is not applicable to this function.
 =over 4
 
 The function C<add_to_xsv> will raise an exception when
-C<< Text::CSV_XS->combine >> fails. Please see the L<Text::CSV_XS(3)|Text::CSV_XS>
+C<< Text::CSV_XS->combine >> fails. Please see the L<Text::CSV_XS|Text::CSV_XS(3)>
 documentation for details about what type of input causes a failure.
 
 =back
 
 =head1 AUTHOR
 
-Evan A. Zacks C<< <evan@commonmind.com> >>
+Evan A. Zacks C<< <zackse@cpan.org> >>
 
 =head1 BUGS
 
@@ -652,7 +705,7 @@ your bug as I make changes.
 
 =head1 SEE ALSO
 
-L<Text::CSV_XS(3)>, L<CGI::Application(3)>
+L<Text::CSV_XS>, L<CGI::Application>
 
 =head1 COPYRIGHT AND LICENSE
 
@@ -663,6 +716,6 @@ and/or modify it under the same terms as Perl itself.
 
 =head1 REVISION
 
-$Id: XSV.pm 26 2005-09-22 15:21:37Z zackse $
+$Id: XSV.pm 31 2005-11-23 15:46:45Z zackse $
 
 =cut
